/***************************************************************************
 *   Copyright (C) 2010 by Weinzierl Stefan   *
 *   Stefan@Weinzierl-Stefan.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "modelviewscene.h"

#include <QGraphicsItem>

ModelViewScene::ModelViewScene(QObject *parent)
 : QGraphicsScene(parent), m_model(0)
{
}

ModelViewScene::~ModelViewScene()
{
}

void ModelViewScene::setModel(QAbstractItemModel *model)
{
	if (m_model) {
		disconnect(m_model, 0, this, 0);
	}

	m_model=model;

	if (m_model) {
		connect(m_model, SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(dataChanged(const QModelIndex&, const QModelIndex&)));
		connect(m_model, SIGNAL(layoutChanged()), this, SLOT(modelReset()));
		connect(m_model, SIGNAL(modelReset()), this, SLOT(modelReset()));
		connect(m_model, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SLOT(rowsInserted(const QModelIndex&, int, int)));
		connect(m_model, SIGNAL(rowsRemoved(const QModelIndex &, int, int)), this, SLOT(rowsRemoved(const QModelIndex&, int, int)));
	}
	modelReset();
}

void ModelViewScene::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
{
	if (topLeft.parent()==m_rootItem && bottomRight.parent()==m_rootItem) {
		for (int row=topLeft.row(); row<=bottomRight.row() && row<m_model->rowCount(m_rootItem); row++) {
			QGraphicsItem *item=m_items.at(row);
			if (item) {
				changeItem(topLeft.sibling(row, 0), item);
			}
		}
	}
}

void ModelViewScene::modelReset()
{
	qDeleteAll(m_items);
	m_items.clear();

	if (m_model) {
		int rows=m_model->rowCount(m_rootItem);

		for (int row=0; row<rows; row++) {
			QGraphicsItem *item=newItem(m_rootItem.child(row, 0));
			m_items.append(item);
			addItem(item);
		}

	}
}

void ModelViewScene::rowsInserted(const QModelIndex &parent, int start, int end)
{
	if (parent==m_rootItem) {
		for (int row=start; row<=end; row++) {
			QGraphicsItem *item=newItem(m_rootItem.child(row, 0));
			m_items.insert(row, item);
			addItem(item);
		}
	}

}

void ModelViewScene::rowsRemoved(const QModelIndex &parent, int start, int end)
{
	if (parent==m_rootItem) {
		for (int row=start; row<=end && row<m_model->rowCount(m_rootItem); row++) {
			delete m_items.takeAt(start);
		}
	}
}

QGraphicsItem *ModelViewScene::newItem(const QModelIndex &index) const
{
	QGraphicsRectItem *item=new QGraphicsRectItem(-30, -10, 60, 20);

	changeItem(index, item);

	return item;
}

void ModelViewScene::changeItem(const QModelIndex &index, QGraphicsItem *item) const
{
	if (index.isValid() && item) {
		int x=index.sibling(index.row(), 0).data().toInt();
		int y=index.sibling(index.row(), 1).data().toInt();
		int rot=index.sibling(index.row(), 2).data().toInt();

		QTransform t;
		t.rotate(rot);
		t.translate(x, y);

		item->setTransform(t);
	}
}

void ModelViewScene::setRootIndex(const QModelIndex &index)
{
	m_rootItem=index;
	modelReset();
}

