/***************************************************************************
 *   Copyright (C); 2010 by Weinzierl Stefan   *
 *   Stefan@Weinzierl-Stefan.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option); any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MODELVIEWSCENE_H
#define MODELVIEWSCENE_H

#include <QGraphicsScene>
#include <QModelIndex>
#include <QPointer>

class QAbstractItemModel;

/**
	@author Weinzierl Stefan <Stefan@Weinzierl-Stefan.de>
*/
class ModelViewScene : public QGraphicsScene
{
Q_OBJECT
public:
	ModelViewScene(QObject *parent=0);
	~ModelViewScene();

	void setModel(QAbstractItemModel *model);

	QAbstractItemModel *model() const { return m_model; };
	QModelIndex rootIndex() const { return m_rootItem; };

public slots:
	void setRootIndex(const QModelIndex &index);

protected:
	QGraphicsItem *newItem(const QModelIndex &index) const;
	void changeItem(const QModelIndex &index, QGraphicsItem *item) const;

private slots:
	void dataChanged(const QModelIndex & topLeft, const QModelIndex & bottomRight);
	void modelReset();
	void rowsInserted(const QModelIndex & parent, int start, int end);
	void rowsRemoved(const QModelIndex & parent, int start, int end);

private:
	QPointer<QAbstractItemModel> m_model;

	QList<QGraphicsItem*> m_items;

	QModelIndex m_rootItem;
};


#endif
